local M = {}

-- HTTP Request class
local Request = {}
Request.__index = Request

function Request.new(url, payload)
    local self = setmetatable({}, Request)
    self.url = url
    self.payload = payload
    return self
end

function Request:execute(callback)
    local client = vim.uv.new_tcp()
    local headers = table.concat({
        string.format("POST %s HTTP/1.1", self.url),
        "Host: generativelanguage.googleapis.com",
        "Content-Type: application/json",
        string.format("Content-Length: %d", #self.payload),
        "Connection: close",
        "",
        self.payload
    }, "\r\n")

    local response_data = ""
    
    client:connect("generativelanguage.googleapis.com", 443, function(err)
        if err then
            vim.schedule(function()
                callback(nil, "Connection failed: " .. err)
            end)
            client:close()
            return
        end

        client:write(headers)
        
        client:read_start(function(err, chunk)
            if err then
                vim.schedule(function()
                    callback(nil, "Read error: " .. err)
                end)
                client:close()
                return
            end

            if chunk then
                response_data = response_data .. chunk
            else
                -- End of response
                client:close()
                
                -- Parse response
                local headers_end = response_data:find("\r\n\r\n")
                if headers_end then
                    local body = response_data:sub(headers_end + 4)
                    local success, decoded = pcall(vim.json.decode, body)
                    
                    vim.schedule(function()
                        if success then
                            callback(decoded)
                        else
                            callback(nil, "JSON decode error: " .. body)
                        end
                    end)
                else
                    vim.schedule(function()
                        callback(nil, "Invalid response format")
                    end)
                end
            end
        end)
    end)
end

M.Request = Request
return M